@@ -27,83 +27,83 @@ ET = webdav.ET
 class SubbedProperty(webdav.Property):
     """Property with sub-components that can be queried."""
 
-    def get_value_ext(self, href, resource, el, environ, requested):
+    async def get_value_ext(self, href, resource, el, environ, requested):
         """Get the value of a data property.
 
-        :param href: Resource href
-        :param resource: Resource to get value for
-        :param el: Element to fill in
-        :param environ: WSGI environ dict
-        :param requested: Requested property (including subelements)
+        Args:
+          href: Resource href
+          resource: Resource to get value for
+          el: Element to fill in
+          environ: WSGI environ dict
+          requested: Requested property (including subelements)
         """
         raise NotImplementedError(self.get_value_ext)
 
 
-def get_properties_with_data(data_property, href, resource, properties,
-                             environ, requested):
+async def get_properties_with_data(
+    data_property, href, resource, properties, environ, requested
+):
     properties = dict(properties)
     properties[data_property.name] = data_property
-    return webdav.get_properties(
-        href, resource, properties, environ, requested)
+    async for ps in webdav.get_properties(
+        href, resource, properties, environ, requested
+    ):
+        yield ps
 
 
 class MultiGetReporter(webdav.Reporter):
     """Abstract base class for multi-get reporters."""
 
-    name = None
+    name: str
 
     # A SubbedProperty subclass
-    data_property = None
+    data_property: SubbedProperty
 
     @webdav.multistatus
-    def report(self, environ, body, resources_by_hrefs, properties, base_href,
-               resource, depth):
+    async def report(
+        self,
+        environ,
+        body,
+        resources_by_hrefs,
+        properties,
+        base_href,
+        resource,
+        depth,
+        strict
+    ):
         # TODO(jelmer): Verify that depth == "0"
         # TODO(jelmer): Verify that resource is an the right resource type
         requested = None
         hrefs = []
         for el in body:
-            if el.tag in ('{DAV:}prop', '{DAV:}allprop', '{DAV:}propname'):
+            if el.tag in ("{DAV:}prop", "{DAV:}allprop", "{DAV:}propname"):
                 requested = el
-            elif el.tag == '{DAV:}href':
+            elif el.tag == "{DAV:}href":
                 hrefs.append(webdav.read_href_element(el))
             else:
-                raise webdav.BadRequestError(
-                    'Unknown tag %s in report %s' % (el.tag, self.name))
+                webdav.nonfatal_bad_request(
+                    f"Unknown tag {el.tag} in report {self.name}",
+                    strict)
+        if requested is None:
+            # The CalDAV RFC says that behaviour mimicks that of PROPFIND,
+            # and the WebDAV RFC says that no body implies {DAV}allprop
+            # This isn't exactly an empty body, but close enough.
+            requested = ET.Element('{DAV:}allprop')
         for (href, resource) in resources_by_hrefs(hrefs):
             if resource is None:
-                yield webdav.Status(href, '404 Not Found', propstat=[])
+                yield webdav.Status(href, "404 Not Found", propstat=[])
             else:
                 propstat = get_properties_with_data(
-                    self.data_property, href, resource, properties, environ,
-                    requested)
-                yield webdav.Status(href, '200 OK', propstat=list(propstat))
+                    self.data_property,
+                    href,
+                    resource,
+                    properties,
+                    environ,
+                    requested,
+                )
+                yield webdav.Status(
+                    href, "200 OK", propstat=[s async for s in propstat]
+                )
 
 
 # see https://tools.ietf.org/html/rfc4790
-
-class UnknownCollation(Exception):
-
-    def __init__(self, collation):
-        super(UnknownCollation, self).__init__(
-            "Collation %r is not supported" % collation)
-        self.collation = collation
-
-
-collations = {
-    'i;ascii-casemap': lambda a, b: (a.decode('ascii').upper() ==
-                                     b.decode('ascii').upper()),
-    'i;octet': lambda a, b: a == b,
-}
-
-
-def get_collation(name):
-    """Get a collation by name.
-
-    :param name: Collation name
-    :raises UnknownCollation: If the collation is not supported
-    """
-    try:
-        return collations[name]
-    except KeyError:
-        raise UnknownCollation(name)
