@@ -17,30 +17,29 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA  02110-1301, USA.
 
-"""ICalendar file handling.
+"""ICalendar file handling."""
 
-"""
-
-import datetime
 import logging
-import pytz
+from collections.abc import Iterable
+from datetime import datetime, time, timedelta, timezone
+from typing import Callable, Optional, Union
 
 import dateutil.rrule
-from icalendar.cal import Calendar, component_factory
-from icalendar.prop import (
-    vDatetime,
-    vDDDTypes,
-    vText,
-)
-from xandikos.store import (
-    Filter,
-    File,
-    InvalidFileContents,
-)
+import pytz
+from icalendar.cal import Calendar, Component, component_factory
+from icalendar.prop import TypesFactory, vCategory, vDatetime, vDDDTypes, vText
+
+from xandikos.store import File, Filter, InvalidFileContents
+
+from . import collation as _mod_collation
+from .store.index import IndexDict, IndexKey, IndexValue, IndexValueIterator
+
+TYPES_FACTORY = TypesFactory()
+
+PropTypes = Union[vText]
+
+TzifyFunction = Callable[[datetime], datetime]
 
-from . import (
-    collation as _mod_collation,
-)
 
 # TODO(jelmer): Populate this further based on
 # https://tools.ietf.org/html/rfc5545#3.3.11
@@ -48,26 +47,32 @@ _INVALID_CONTROL_CHARACTERS = ["\x0c", "\x01"]
 
 
 class MissingProperty(Exception):
-    def __init__(self, property_name):
-        super(MissingProperty, self).__init__("Property %r missing" % property_name)
+    def __init__(self, property_name) -> None:
+        super().__init__(
+            f"Property {property_name!r} missing")
         self.property_name = property_name
 
 
 def validate_calendar(cal, strict=False):
     """Validate a calendar object.
 
-    :param cal: Calendar object
-    :return: iterator over error messages
+    Args:
+      cal: Calendar object
+    Returns: iterator over error messages
     """
-    for error in validate_component(cal, strict=strict):
-        yield error
+    yield from validate_component(cal, strict=strict)
 
 
-def create_subindexes(indexes, base):
-    ret = {}
+# SubIndexDict is like IndexDict, but None can also occur as a key
+SubIndexDict = dict[Optional[IndexKey], IndexValue]
+
+
+def create_subindexes(
+        indexes: Union[SubIndexDict, IndexDict], base: str) -> SubIndexDict:
+    ret: SubIndexDict = {}
     for k, v in indexes.items():
         if k is not None and k.startswith(base + "/"):
-            ret[k[len(base) + 1 :]] = v
+            ret[k[len(base) + 1:]] = v
         elif k == base:
             ret[None] = v
     return ret
@@ -76,14 +81,15 @@ def create_subindexes(indexes, base):
 def validate_component(comp, strict=False):
     """Validate a calendar component.
 
-    :param comp: Calendar component
+    Args:
+      comp: Calendar component
     """
     # Check text fields for invalid characters
     for (name, value) in comp.items():
         if isinstance(value, vText):
             for c in _INVALID_CONTROL_CHARACTERS:
                 if c in value:
-                    yield "Invalid character %s in field %s" % (
+                    yield "Invalid character {} in field {}".format(
                         c.encode("unicode_escape"),
                         name,
                     )
@@ -92,18 +98,18 @@ def validate_component(comp, strict=False):
             try:
                 comp[required]
             except KeyError:
-                yield "Missing required field %s" % required
+                yield f"Missing required field {required}"
     for subcomp in comp.subcomponents:
-        for error in validate_component(subcomp, strict=strict):
-            yield error
+        yield from validate_component(subcomp, strict=strict)
 
 
 def calendar_component_delta(old_cal, new_cal):
     """Find the differences between components in two calendars.
 
-    :param old_cal: Old calendar (can be None)
-    :param new_cal: New calendar (can be None)
-    :yield: (old_component, new_component) tuples (either can be None)
+    Args:
+      old_cal: Old calendar (can be None)
+      new_cal: New calendar (can be None)
+    Returns: iterator over (old_component, new_component) tuples (either can be None)
     """
     by_uid = {}
     by_content = {}
@@ -153,7 +159,7 @@ def calendar_prop_delta(old_component, new_component):
 def describe_component(component):
     if component.name == "VTODO":
         try:
-            return "task '%s'" % component["SUMMARY"]
+            return f"task '{component['SUMMARY']}'"
         except KeyError:
             return "task"
     else:
@@ -163,61 +169,62 @@ def describe_component(component):
             return "calendar item"
 
 
-DELTA_IGNORE_FIELDS = set(
-    [
-        "LAST-MODIFIED",
-        "SEQUENCE",
-        "DTSTAMP",
-        "PRODID",
-        "CREATED",
-        "COMPLETED",
-        "X-MOZ-GENERATION",
-        "X-LIC-ERROR",
-        "UID",
-    ]
-)
+DELTA_IGNORE_FIELDS = {
+    "LAST-MODIFIED",
+    "SEQUENCE",
+    "DTSTAMP",
+    "PRODID",
+    "CREATED",
+    "COMPLETED",
+    "X-MOZ-GENERATION",
+    "X-LIC-ERROR",
+    "UID",
+}
 
 
 def describe_calendar_delta(old_cal, new_cal):
     """Describe the differences between two calendars.
 
-    :param old_cal: Old calendar (can be None)
-    :param new_cal: New calendar (can be None)
-    :yield: Lines describing changes
+    Args:
+      old_cal: Old calendar (can be None)
+      new_cal: New calendar (can be None)
+    Returns: Lines describing changes
     """
     # TODO(jelmer): Extend
-    for old_component, new_component in calendar_component_delta(old_cal, new_cal):
+    for old_component, new_component in calendar_component_delta(
+            old_cal, new_cal):
         if not new_component:
-            yield "Deleted %s" % describe_component(old_component)
+            yield f"Deleted {describe_component(old_component)}"
             continue
         description = describe_component(new_component)
         if not old_component:
-            yield "Added %s" % describe_component(new_component)
+            yield f"Added {describe_component(new_component)}"
             continue
         for field, old_value, new_value in calendar_prop_delta(
             old_component, new_component
         ):
             if field.upper() in DELTA_IGNORE_FIELDS:
                 continue
-            if old_component.name.upper() == "VTODO" and field.upper() == "STATUS":
+            if (old_component.name.upper() == "VTODO"
+                    and field.upper() == "STATUS"):
                 if new_value is None:
-                    yield "status of %s deleted" % description
+                    yield f"status of {description} deleted"
                 else:
                     human_readable = {
                         "NEEDS-ACTION": "needing action",
                         "COMPLETED": "complete",
                         "CANCELLED": "cancelled",
                     }
-                    yield "%s marked as %s" % (
+                    yield "{} marked as {}".format(
                         description,
                         human_readable.get(new_value.upper(), new_value),
                     )
             elif field.upper() == "DESCRIPTION":
-                yield "changed description of %s" % description
+                yield f"changed description of {description}"
             elif field.upper() == "SUMMARY":
-                yield "changed summary of %s" % description
+                yield f"changed summary of {description}"
             elif field.upper() == "LOCATION":
-                yield "changed location of %s to %s" % (description, new_value)
+                yield f"changed location of {description} to {new_value}"
             elif (
                 old_component.name.upper() == "VTODO"
                 and field.upper() == "PERCENT-COMPLETE"
@@ -225,31 +232,31 @@ def describe_calendar_delta(old_cal, new_cal):
             ):
                 yield "%s marked as %d%% completed." % (description, new_value)
             elif field.upper() == "DUE":
-                yield "changed due date for %s from %s to %s" % (
+                yield "changed due date for {} from {} to {}".format(
                     description,
                     old_value.dt if old_value else "none",
                     new_value.dt if new_value else "none",
                 )
             elif field.upper() == "DTSTART":
-                yield "changed start date/time of %s from %s to %s" % (
+                yield "changed start date/time of {} from {} to {}".format(
                     description,
                     old_value.dt if old_value else "none",
                     new_value.dt if new_value else "none",
                 )
             elif field.upper() == "DTEND":
-                yield "changed end date/time of %s from %s to %s" % (
+                yield "changed end date/time of {} from {} to {}".format(
                     description,
                     old_value.dt if old_value else "none",
                     new_value.dt if new_value else "none",
                 )
             elif field.upper() == "CLASS":
-                yield "changed class of %s from %s to %s" % (
+                yield "changed class of {} from {} to {}".format(
                     description,
                     old_value.lower() if old_value else "none",
                     new_value.lower() if new_value else "none",
                 )
             else:
-                yield "modified field %s in %s" % (field, description)
+                yield f"modified field {field} in {description}"
                 logging.debug(
                     "Changed %s/%s or %s/%s from %s to %s.",
                     old_component.name,
@@ -281,7 +288,7 @@ def apply_time_range_vevent(start, end, comp, tzify):
     if getattr(dtstart.dt, "time", None) is not None:
         return start <= tzify(dtstart.dt)
     else:
-        return start < (tzify(dtstart.dt) + datetime.timedelta(1))
+        return start < (tzify(dtstart.dt) + timedelta(1))
 
 
 def apply_time_range_vjournal(start, end, comp, tzify):
@@ -295,7 +302,7 @@ def apply_time_range_vjournal(start, end, comp, tzify):
     if getattr(dtstart.dt, "time", None) is not None:
         return start <= tzify(dtstart.dt)
     else:
-        return start < (tzify(dtstart.dt) + datetime.timedelta(1))
+        return start < (tzify(dtstart.dt) + timedelta(1))
 
 
 def apply_time_range_vtodo(start, end, comp, tzify):
@@ -307,7 +314,8 @@ def apply_time_range_vtodo(start, end, comp, tzify):
         duration = comp.get("DURATION")
         if duration and not due:
             return start <= tzify(dtstart.dt) + duration.dt and (
-                end > tzify(dtstart.dt) or end >= tzify(dtstart.dt) + duration.dt
+                end > tzify(dtstart.dt)
+                or end >= tzify(dtstart.dt) + duration.dt
             )
         elif due and not duration:
             return (start <= tzify(dtstart.dt) or start < tzify(due.dt)) and (
@@ -323,9 +331,9 @@ def apply_time_range_vtodo(start, end, comp, tzify):
     created = comp.get("CREATED")
     if completed:
         if created:
-            return (start <= tzify(created.dt) or start <= tzify(completed.dt)) and (
-                end >= tzify(created.dt) or end >= tzify(completed.dt)
-            )
+            return (start <= tzify(created.dt)
+                    or start <= tzify(completed.dt)) and (
+                        end >= tzify(created.dt) or end >= tzify(completed.dt))
         else:
             return start <= tzify(completed.dt) and end >= tzify(completed.dt)
     elif created:
@@ -351,25 +359,30 @@ def apply_time_range_valarm(start, end, comp, tzify):
     raise NotImplementedError(apply_time_range_valarm)
 
 
-class PropertyTimeRangeMatcher(object):
-    def __init__(self, start, end):
+class PropertyTimeRangeMatcher:
+    def __init__(self, start: datetime, end: datetime) -> None:
         self.start = start
         self.end = end
 
-    def __repr__(self):
-        return "%s(%r, %r)" % (self.__class__.__name__, self.start, self.end)
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}({self.start!r}, {self.end!r})"
 
     def match(self, prop, tzify):
         dt = tzify(prop.dt)
         return dt >= self.start and dt <= self.end
 
-    def match_indexes(self, prop, tzify):
+    def match_indexes(self, prop: SubIndexDict, tzify: TzifyFunction):
         return any(
-            self.match(vDDDTypes(vDatetime.from_ical(p)), tzify) for p in prop[None]
-        )
+            self.match(
+                vDDDTypes(vDDDTypes.from_ical(p.decode('utf-8'))), tzify)
+            for p in prop[None] if not isinstance(p, bool))
 
 
-class ComponentTimeRangeMatcher(object):
+TimeRangeFilter = Callable[
+    [datetime, datetime, Component, TzifyFunction], bool]
+
+
+class ComponentTimeRangeMatcher:
 
     all_props = [
         "DTSTART",
@@ -383,7 +396,7 @@ class ComponentTimeRangeMatcher(object):
 
     # According to https://tools.ietf.org/html/rfc4791, section 9.9 these
     # are the properties to check.
-    component_handlers = {
+    component_handlers: dict[str, TimeRangeFilter] = {
         "VEVENT": apply_time_range_vevent,
         "VTODO": apply_time_range_vtodo,
         "VJOURNAL": apply_time_range_vjournal,
@@ -391,52 +404,58 @@ class ComponentTimeRangeMatcher(object):
         "VALARM": apply_time_range_valarm,
     }
 
-    def __init__(self, start, end, comp=None):
+    def __init__(self, start, end, comp=None) -> None:
         self.start = start
         self.end = end
         self.comp = comp
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         if self.comp is not None:
-            return "%s(%r, %r, comp=%r)" % (
+            return "{}({!r}, {!r}, comp={!r})".format(
                 self.__class__.__name__,
                 self.start,
                 self.end,
                 self.comp,
             )
         else:
-            return "%s(%r, %r)" % (
-                self.__class__.__name__,
-                self.start,
-                self.end,
-            )
+            return f"{self.__class__.__name__}({self.start!r}, {self.end!r})"
 
-    def match(self, comp, tzify):
+    def match(self, comp: Component, tzify: TzifyFunction):
         try:
             component_handler = self.component_handlers[comp.name]
         except KeyError:
-            logging.warning("unknown component %r in time-range filter", comp.name)
+            logging.warning(
+                "unknown component %r in time-range filter", comp.name)
             return False
         return component_handler(self.start, self.end, comp, tzify)
 
-    def match_indexes(self, indexes, tzify):
-        vs = {}
-        for name, value in indexes.items():
-            if name and name[2:] in self.all_props:
-                if value and value[0]:
-                    if not isinstance(value[0], vDDDTypes):
-                        vs[name[2:]] = vDDDTypes(vDatetime.from_ical(value[0]))
-                    else:
-                        vs[name[2:]] = value[0]
+    def match_indexes(self, indexes: SubIndexDict, tzify: TzifyFunction):
+        vs: dict[str, vDDDTypes] = {}
+        for name, values in indexes.items():
+            if not name:
+                continue
+            field = name[2:]
+            if field not in self.all_props:
+                continue
+            for value in values:
+                if value and not isinstance(value, bool):
+                    vs.setdefault(field, []).append(
+                        vDDDTypes(vDDDTypes.from_ical(value.decode('utf-8'))))
 
         try:
             component_handler = self.component_handlers[self.comp]
         except KeyError:
-            logging.warning("unknown component %r in time-range filter", self.comp)
+            logging.warning(
+                "unknown component %r in time-range filter", self.comp)
             return False
-        return component_handler(self.start, self.end, vs, tzify)
+        return component_handler(
+            self.start,
+            self.end,
+            # TODO(jelmer): What to do if there is more than one value?
+            {k: vs[0] for (k, vs) in vs.items()},
+            tzify)
 
-    def index_keys(self):
+    def index_keys(self) -> list[list[str]]:
         if self.comp == "VEVENT":
             props = ["DTSTART", "DTEND", "DURATION"]
         elif self.comp == "VTODO":
@@ -452,73 +471,95 @@ class ComponentTimeRangeMatcher(object):
         return [["P=" + prop] for prop in props]
 
 
-class TextMatcher(object):
-    def __init__(self, text, collation=None, negate_condition=False):
-        if isinstance(text, str):
-            text = text.encode()
+class TextMatcher:
+    def __init__(self, name: str, text: str,
+                 collation: Optional[str] = None,
+                 negate_condition: bool = False) -> None:
+        self.name = name
+        self.type_fn = TYPES_FACTORY.for_property(name)
+        assert isinstance(text, str)
         self.text = text
         if collation is None:
             collation = "i;ascii-casemap"
         self.collation = _mod_collation.get_collation(collation)
         self.negate_condition = negate_condition
 
-    def __repr__(self):
-        return "%s(%r, collation=%r, negate_condition=%r)" % (
+    def __repr__(self) -> str:
+        return "{}({!r}, {!r}, collation={!r}, negate_condition={!r})".format(
             self.__class__.__name__,
+            self.name,
             self.text,
             self.collation,
             self.negate_condition,
         )
 
-    def match_indexes(self, indexes):
-        return any(self.match(k) for k in indexes[None])
+    def match_indexes(self, indexes: SubIndexDict):
+        return any(
+            self.match(self.type_fn(self.type_fn.from_ical(k)))
+            for k in indexes[None])
 
-    def match(self, prop):
+    def match(self, prop: Union[vText, vCategory, str]):
         if isinstance(prop, vText):
-            prop = prop.encode()
-        matches = self.collation(self.text, prop)
+            matches = self.collation(self.text, str(prop), 'equals')
+        elif isinstance(prop, str):
+            matches = self.collation(self.text, prop, 'equals')
+        elif isinstance(prop, vCategory):
+            matches = any([self.match(cat) for cat in prop.cats])
+        else:
+            logging.warning(
+                "potentially unsupported value in text match search: " +
+                repr(prop))
+            return False
         if self.negate_condition:
             return not matches
         else:
             return matches
 
 
-class ComponentFilter(object):
-    def __init__(self, name, children=None, is_not_defined=False, time_range=None):
+class ComponentFilter:
+
+    time_range: Optional[ComponentTimeRangeMatcher]
+
+    def __init__(
+            self, name: str, children=None, is_not_defined: bool = False,
+            time_range=None) -> None:
         self.name = name
         self.children = children
         self.is_not_defined = is_not_defined
         self.time_range = time_range
         self.children = children or []
 
-    def __repr__(self):
-        return "%s(%r, children=%r, is_not_defined=%r, time_range=%r)" % (
-            self.__class__.__name__,
-            self.name,
-            self.children,
-            self.is_not_defined,
-            self.time_range,
-        )
+    def __repr__(self) -> str:
+        return ("{}({!r}, children={!r}, is_not_defined={!r}, time_range={!r})"
+                .format(
+                    self.__class__.__name__,
+                    self.name,
+                    self.children,
+                    self.is_not_defined,
+                    self.time_range))
 
-    def filter_subcomponent(self, name, is_not_defined=False, time_range=None):
+    def filter_subcomponent(
+            self, name: str, is_not_defined: bool = False,
+            time_range: Optional[ComponentTimeRangeMatcher] = None):
         ret = ComponentFilter(
             name=name, is_not_defined=is_not_defined, time_range=time_range
         )
         self.children.append(ret)
         return ret
 
-    def filter_property(self, name, is_not_defined=False, time_range=None):
+    def filter_property(self, name: str, is_not_defined: bool = False,
+                        time_range: Optional[PropertyTimeRangeMatcher] = None):
         ret = PropertyFilter(
             name=name, is_not_defined=is_not_defined, time_range=time_range
         )
         self.children.append(ret)
         return ret
 
-    def filter_time_range(self, start, end):
+    def filter_time_range(self, start: datetime, end: datetime):
         self.time_range = ComponentTimeRangeMatcher(start, end, comp=self.name)
         return self.time_range
 
-    def match(self, comp, tzify):
+    def match(self, comp: Component, tzify: TzifyFunction):
         # From https://tools.ietf.org/html/rfc4791, 9.7.1:
         # A CALDAV:comp-filter is said to match if:
 
@@ -538,7 +579,8 @@ class ComponentFilter(object):
         # 3. The CALDAV:comp-filter XML element contains a CALDAV:time-range
         # XML element and at least one recurrence instance in the targeted
         # calendar component is scheduled to overlap the specified time range
-        if self.time_range is not None and not self.time_range.match(comp, tzify):
+        if (self.time_range is not None
+                and not self.time_range.match(comp, tzify)):
             return False
 
         # ... and all specified CALDAV:prop-filter and CALDAV:comp-filter child
@@ -557,10 +599,11 @@ class ComponentFilter(object):
 
     def _implicitly_defined(self):
         return any(
-            not getattr(child, "is_not_defined", False) for child in self.children
+            not getattr(child, "is_not_defined", False)
+            for child in self.children
         )
 
-    def match_indexes(self, indexes, tzify):
+    def match_indexes(self, indexes: IndexDict, tzify: TzifyFunction):
         myindex = "C=" + self.name
         if self.is_not_defined:
             return not bool(indexes[myindex])
@@ -583,44 +626,51 @@ class ComponentFilter(object):
 
     def index_keys(self):
         mine = "C=" + self.name
-        for child in self.children + ([self.time_range] if self.time_range else []):
+        for child in (
+                self.children +
+                ([self.time_range] if self.time_range else [])):
             for tl in child.index_keys():
                 yield [(mine + "/" + child_index) for child_index in tl]
         if not self._implicitly_defined():
             yield [mine]
 
 
-class PropertyFilter(object):
-    def __init__(self, name, children=None, is_not_defined=False, time_range=None):
+class PropertyFilter:
+
+    def __init__(self, name: str, children=None, is_not_defined: bool = False,
+                 time_range: Optional[PropertyTimeRangeMatcher] = None) -> None:
         self.name = name
         self.is_not_defined = is_not_defined
         self.children = children or []
         self.time_range = time_range
 
-    def __repr__(self):
-        return "%s(%r, children=%r, is_not_defined=%r, time_range=%r)" % (
-            self.__class__.__name__,
-            self.name,
-            self.children,
-            self.is_not_defined,
-            self.time_range,
-        )
+    def __repr__(self) -> str:
+        return ("{}({!r}, children={!r}, is_not_defined={!r}, time_range={!r})"
+                .format(
+                    self.__class__.__name__, self.name, self.children,
+                    self.is_not_defined, self.time_range))
 
-    def filter_parameter(self, name, is_not_defined=False):
+    def filter_parameter(
+            self, name: str,
+            is_not_defined: bool = False) -> "ParameterFilter":
         ret = ParameterFilter(name=name, is_not_defined=is_not_defined)
         self.children.append(ret)
         return ret
 
-    def filter_time_range(self, start, end):
+    def filter_time_range(
+            self, start: datetime, end: datetime) -> PropertyTimeRangeMatcher:
         self.time_range = PropertyTimeRangeMatcher(start, end)
         return self.time_range
 
-    def filter_text_match(self, text, collation=None, negate_condition=False):
-        ret = TextMatcher(text, collation=collation, negate_condition=negate_condition)
+    def filter_text_match(
+            self, text: str, collation: Optional[str] = None,
+            negate_condition: bool = False) -> TextMatcher:
+        ret = TextMatcher(self.name, text, collation=collation,
+                          negate_condition=negate_condition)
         self.children.append(ret)
         return ret
 
-    def match(self, comp, tzify):
+    def match(self, comp: Component, tzify: TzifyFunction) -> bool:
         # From https://tools.ietf.org/html/rfc4791, 9.7.2:
         # A CALDAV:comp-filter is said to match if:
 
@@ -645,11 +695,13 @@ class PropertyFilter(object):
 
         return True
 
-    def match_indexes(self, indexes, tzify):
+    def match_indexes(
+            self, indexes: SubIndexDict,
+            tzify: TzifyFunction) -> bool:
         myindex = "P=" + self.name
         if self.is_not_defined:
             return not bool(indexes[myindex])
-        subindexes = create_subindexes(indexes, myindex)
+        subindexes: SubIndexDict = create_subindexes(indexes, myindex)
         if not self.children and not self.time_range:
             return bool(indexes[myindex])
 
@@ -674,23 +726,30 @@ class PropertyFilter(object):
         yield [mine]
 
 
-class ParameterFilter(object):
-    def __init__(self, name, children=None, is_not_defined=False):
+class ParameterFilter:
+
+    children: list[TextMatcher]
+
+    def __init__(self, name: str, children: Optional[list[TextMatcher]] = None,
+                 is_not_defined: bool = False) -> None:
         self.name = name
         self.is_not_defined = is_not_defined
         self.children = children or []
 
-    def filter_text_match(self, text, collation=None, negate_condition=False):
-        ret = TextMatcher(text, collation=collation, negate_condition=negate_condition)
+    def filter_text_match(self, text: str, collation: Optional[str] = None,
+                          negate_condition: bool = False) -> TextMatcher:
+        ret = TextMatcher(
+            self.name, text, collation=collation,
+            negate_condition=negate_condition)
         self.children.append(ret)
         return ret
 
-    def match(self, prop):
+    def match(self, prop: PropTypes) -> bool:
         if self.is_not_defined:
             return self.name not in prop.params
 
         try:
-            value = prop.params[self.name].encode()
+            value = prop.params[self.name]
         except KeyError:
             return False
 
@@ -699,21 +758,21 @@ class ParameterFilter(object):
                 return False
         return True
 
-    def index_keys(self):
+    def index_keys(self) -> Iterable[list[str]]:
         yield ["A=" + self.name]
 
-    def match_indexes(self, indexes):
+    def match_indexes(self, indexes: IndexDict) -> bool:
         myindex = "A=" + self.name
         if self.is_not_defined:
             return not bool(indexes[myindex])
 
-        try:
-            value = indexes[myindex][0]
-        except IndexError:
+        subindexes = create_subindexes(indexes, myindex)
+
+        if not subindexes:
             return False
 
         for child in self.children:
-            if not child.match(value):
+            if not child.match_indexes(subindexes):
                 return False
         return True
 
@@ -723,9 +782,9 @@ class CalendarFilter(Filter):
 
     content_type = "text/calendar"
 
-    def __init__(self, default_timezone):
+    def __init__(self, default_timezone: Union[str, timezone]) -> None:
         self.tzify = lambda dt: as_tz_aware_ts(dt, default_timezone)
-        self.children = []
+        self.children: list[ComponentFilter] = []
 
     def filter_subcomponent(self, name, is_not_defined=False, time_range=None):
         ret = ComponentFilter(
@@ -734,7 +793,10 @@ class CalendarFilter(Filter):
         self.children.append(ret)
         return ret
 
-    def check(self, name, file):
+    def check(self, name: str, file: File) -> bool:
+        if not isinstance(file, ICalendarFile):
+            return False
+
         c = file.calendar
         if c is None:
             return False
@@ -753,7 +815,7 @@ class CalendarFilter(Filter):
                 return False
         return True
 
-    def check_from_indexes(self, name, indexes):
+    def check_from_indexes(self, name: str, indexes: IndexDict) -> bool:
         for child_filter in self.children:
             try:
                 if not child_filter.match_indexes(indexes, self.tzify):
@@ -768,14 +830,14 @@ class CalendarFilter(Filter):
                 return False
         return True
 
-    def index_keys(self):
+    def index_keys(self) -> list[str]:
         subindexes = []
         for child in self.children:
             subindexes.extend(child.index_keys())
         return subindexes
 
-    def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self.children)
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}({self.children!r})"
 
 
 class ICalendarFile(File):
@@ -783,11 +845,11 @@ class ICalendarFile(File):
 
     content_type = "text/calendar"
 
-    def __init__(self, content, content_type):
-        super(ICalendarFile, self).__init__(content, content_type)
+    def __init__(self, content, content_type) -> None:
+        super().__init__(content, content_type)
         self._calendar = None
 
-    def validate(self):
+    def validate(self) -> None:
         """Verify that file contents are valid."""
         cal = self.calendar
         # TODO(jelmer): return the list of errors to the caller
@@ -810,8 +872,9 @@ class ICalendarFile(File):
         if self._calendar is None:
             try:
                 self._calendar = Calendar.from_ical(b"".join(self.content))
-            except ValueError as e:
-                raise InvalidFileContents(self.content_type, self.content, str(e))
+            except ValueError as exc:
+                raise InvalidFileContents(
+                    self.content_type, self.content, str(exc)) from exc
         return self._calendar
 
     def describe_delta(self, name, previous):
@@ -824,7 +887,7 @@ class ICalendarFile(File):
         except NotImplementedError:
             lines = []
         if not lines:
-            lines = super(ICalendarFile, self).describe_delta(name, previous)
+            lines = super().describe_delta(name, previous)
         return lines
 
     def describe(self, name):
@@ -838,13 +901,14 @@ class ICalendarFile(File):
                     return component["SUMMARY"]
                 except KeyError:
                     pass
-        return super(ICalendarFile, self).describe(name)
+        return super().describe(name)
 
     def get_uid(self):
         """Extract the UID from a VCalendar file.
 
-        :param cal: Calendar, possibly serialized.
-        :return: UID
+        Args:
+          cal: Calendar, possibly serialized.
+        Returns: UID
         """
         for component in self.calendar.subcomponents:
             try:
@@ -853,15 +917,17 @@ class ICalendarFile(File):
                 pass
         raise KeyError
 
-    def _get_index(self, key):
+    def _get_index(self, key: IndexKey) -> IndexValueIterator:
         todo = [(self.calendar, key.split("/"))]
         rest = []
+        c: Component
         while todo:
             (c, segments) = todo.pop(0)
             if segments and segments[0].startswith("C="):
                 if c.name == segments[0][2:]:
                     if len(segments) > 1 and segments[1].startswith("C="):
-                        todo.extend((comp, segments[1:]) for comp in c.subcomponents)
+                        todo.extend(
+                            (comp, segments[1:]) for comp in c.subcomponents)
                     else:
                         rest.append((c, segments[1:]))
 
@@ -871,30 +937,31 @@ class ICalendarFile(File):
             elif segments[0].startswith("P="):
                 assert len(segments) == 1
                 try:
-                    yield c[segments[0][2:]]
+                    p = c[segments[0][2:]]
                 except KeyError:
                     pass
+                else:
+                    if p is not None:
+                        yield p.to_ical()
             else:
-                raise AssertionError("segments: %r" % segments)
+                raise AssertionError(f"segments: {segments!r}")
 
 
-def as_tz_aware_ts(dt, default_timezone):
+def as_tz_aware_ts(dt, default_timezone: Union[str, timezone]) -> datetime:
     if not getattr(dt, "time", None):
-        dt = datetime.datetime.combine(dt, datetime.time())
+        dt = datetime.combine(dt, time())
     if dt.tzinfo is None:
         dt = dt.replace(tzinfo=default_timezone)
     assert dt.tzinfo
     return dt
 
 
-def rruleset_from_comp(comp):
-    if "RRULE" not in comp:
-        return None
+def rruleset_from_comp(comp: Component) -> dateutil.rrule.rruleset:
     dtstart = comp["DTSTART"].dt
     rrulestr = comp["RRULE"].to_ical().decode("utf-8")
     rrule = dateutil.rrule.rrulestr(rrulestr, dtstart=dtstart)
     rs = dateutil.rrule.rruleset()
-    rs.rrule(rrule)
+    rs.rrule(rrule)  # type: ignore
     if "EXDATE" in comp:
         for exdate in comp["EXDATE"]:
             rs.exdate(exdate)
@@ -908,7 +975,11 @@ def rruleset_from_comp(comp):
     return rs
 
 
-def _expand_rrule_component(incomp, start, end, existing):
+def _expand_rrule_component(
+        incomp: Component, start: datetime, end: datetime,
+        existing: dict[str, Component]) -> Iterable[Component]:
+    if "RRULE" not in incomp:
+        return
     rs = rruleset_from_comp(incomp)
     for field in ["RRULE", "EXRULE", "UNTIL", "RDATE", "EXDATE"]:
         if field in incomp:
@@ -926,10 +997,12 @@ def _expand_rrule_component(incomp, start, end, existing):
         yield outcomp
 
 
-def expand_calendar_rrule(incal, start, end):
+def expand_calendar_rrule(
+        incal: Calendar, start: datetime, end: datetime) -> Calendar:
     outcal = Calendar()
     if incal.name != "VCALENDAR":
-        raise AssertionError("called on file with root component %s" % incal.name)
+        raise AssertionError(
+            f"called on file with root component {incal.name}")
     for field in incal:
         outcal[field] = incal[field]
     known = {}
