@@ -17,20 +17,18 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA  02110-1301, USA.
 
-"""Collection configuration file.
-"""
+"""Collection configuration file."""
 
 import configparser
 
-FILENAME = '.xandikos'
+FILENAME = ".xandikos"
 
 
-class CollectionMetadata(object):
+class CollectionMetadata:
     """Metadata for a configuration."""
 
     def get_color(self) -> str:
-        """Get the color for this collection.
-        """
+        """Get the color for this collection."""
         raise NotImplementedError(self.get_color)
 
     def set_color(self, color: str) -> None:
@@ -38,13 +36,11 @@ class CollectionMetadata(object):
         raise NotImplementedError(self.set_color)
 
     def get_source_url(self) -> str:
-        """Get the source URL for this collection.
-        """
+        """Get the source URL for this collection."""
         raise NotImplementedError(self.get_source_url)
 
     def set_source_url(self, url: str) -> None:
-        """Set the source URL for this collection.
-        """
+        """Set the source URL for this collection."""
         raise NotImplementedError(self.set_source_url)
 
     def get_comment(self) -> str:
@@ -66,7 +62,7 @@ class CollectionMetadata(object):
 class FileBasedCollectionMetadata(CollectionMetadata):
     """Metadata for a configuration."""
 
-    def __init__(self, cp=None, save=None):
+    def __init__(self, cp=None, save=None) -> None:
         if cp is None:
             cp = configparser.ConfigParser()
         self._configparser = cp
@@ -84,71 +80,71 @@ class FileBasedCollectionMetadata(CollectionMetadata):
         return cls(cp)
 
     def get_source_url(self):
-        return self._configparser['DEFAULT']['source']
+        return self._configparser["DEFAULT"]["source"]
 
     def set_source_url(self, url):
         if url is not None:
-            self._configparser['DEFAULT']['source'] = url
+            self._configparser["DEFAULT"]["source"] = url
         else:
-            del self._configparser['DEFAULT']['source']
+            del self._configparser["DEFAULT"]["source"]
         self._save("Set source URL.")
 
     def get_color(self):
-        return self._configparser['DEFAULT']['color']
+        return self._configparser["DEFAULT"]["color"]
 
     def get_comment(self):
-        return self._configparser['DEFAULT']['comment']
+        return self._configparser["DEFAULT"]["comment"]
 
     def get_displayname(self):
-        return self._configparser['DEFAULT']['displayname']
+        return self._configparser["DEFAULT"]["displayname"]
 
     def get_description(self):
-        return self._configparser['DEFAULT']['description']
+        return self._configparser["DEFAULT"]["description"]
 
     def set_color(self, color):
         if color is not None:
-            self._configparser['DEFAULT']['color'] = color
+            self._configparser["DEFAULT"]["color"] = color
         else:
-            del self._configparser['DEFAULT']['color']
+            del self._configparser["DEFAULT"]["color"]
         self._save("Set color.")
 
     def set_displayname(self, displayname):
         if displayname is not None:
-            self._configparser['DEFAULT']['displayname'] = displayname
+            self._configparser["DEFAULT"]["displayname"] = displayname
         else:
-            del self._configparser['DEFAULT']['displayname']
+            del self._configparser["DEFAULT"]["displayname"]
         self._save("Set display name.")
 
     def set_description(self, description):
         if description is not None:
-            self._configparser['DEFAULT']['description'] = description
+            self._configparser["DEFAULT"]["description"] = description
         else:
-            del self._configparser['DEFAULT']['description']
+            del self._configparser["DEFAULT"]["description"]
         self._save("Set description.")
 
     def set_comment(self, comment):
         if comment is not None:
-            self._configparser['DEFAULT']['comment'] = comment
+            self._configparser["DEFAULT"]["comment"] = comment
         else:
-            del self._configparser['DEFAULT']['comment']
+            del self._configparser["DEFAULT"]["comment"]
         self._save("Set comment.")
 
     def set_type(self, store_type):
-        self._configparser['DEFAULT']['type'] = store_type
+        self._configparser["DEFAULT"]["type"] = store_type
         self._save("Set collection type.")
 
     def get_type(self):
-        return self._configparser['DEFAULT']['type']
+        return self._configparser["DEFAULT"]["type"]
 
     def get_order(self):
-        return self._configparser['calendar']['order']
+        return self._configparser["calendar"]["order"]
 
     def set_order(self, order):
         try:
-            self._configparser.add_section('calendar')
+            self._configparser.add_section("calendar")
         except configparser.DuplicateSectionError:
             pass
         if order is None:
-            del self._configparser['calendar']['order']
+            del self._configparser["calendar"]["order"]
         else:
-            self._configparser['calendar']['order'] = order
+            self._configparser["calendar"]["order"] = order
