@@ -23,30 +23,19 @@ See https://github.com/pimutils/vdirsyncer/blob/master/docs/vdir.rst
 """
 
 import configparser
-import errno
 import hashlib
 import logging
 import os
 import shutil
+from typing import Dict
 import uuid
 
-from . import (
-    MIMETYPES,
-    Store,
-    DuplicateUidError,
-    InvalidETag,
-    InvalidFileContents,
-    NoSuchItem,
-    open_by_content_type,
-    open_by_extension,
-)
-from .config import (
-    FileBasedCollectionMetadata,
-    FILENAME as CONFIG_FILENAME,
-)
+from . import (MIMETYPES, DuplicateUidError, InvalidETag, InvalidFileContents,
+               NoSuchItem, Store, open_by_content_type, open_by_extension)
+from .config import FILENAME as CONFIG_FILENAME
+from .config import FileBasedCollectionMetadata
 from .index import MemoryIndex
 
-
 DEFAULT_ENCODING = "utf-8"
 
 
@@ -56,14 +45,14 @@ logger = logging.getLogger(__name__)
 class VdirStore(Store):
     """A Store backed by a Vdir directory."""
 
-    def __init__(self, path, check_for_duplicate_uids=True):
-        super(VdirStore, self).__init__(MemoryIndex())
+    def __init__(self, path, check_for_duplicate_uids=True) -> None:
+        super().__init__(MemoryIndex())
         self.path = path
         self._check_for_duplicate_uids = check_for_duplicate_uids
         # Set of blob ids that have already been scanned
-        self._fname_to_uid = {}
+        self._fname_to_uid: Dict[str, str] = {}
         # Maps uids to (sha, fname)
-        self._uid_to_fname = {}
+        self._uid_to_fname: Dict[str, str] = {}
         cp = configparser.ConfigParser()
         cp.read([os.path.join(self.path, CONFIG_FILENAME)])
 
@@ -73,8 +62,8 @@ class VdirStore(Store):
 
         self.config = FileBasedCollectionMetadata(cp, save=save_config)
 
-    def __repr__(self):
-        return "%s(%r)" % (type(self).__name__, self.path)
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.path!r})"
 
     def _get_etag(self, name):
         path = os.path.join(self.path, name)
@@ -83,34 +72,36 @@ class VdirStore(Store):
             with open(path, "rb") as f:
                 for chunk in f:
                     md5.update(chunk)
-        except IOError as e:
-            if e.errno == errno.ENOENT:
-                raise KeyError
-            raise
+        except FileNotFoundError as exc:
+            raise KeyError(name) from exc
+        except IsADirectoryError as exc:
+            raise KeyError(name) from exc
         return md5.hexdigest()
 
     def _get_raw(self, name, etag=None):
         """Get the raw contents of an object.
 
-        :param name: Name of the item
-        :param etag: Optional etag (ignored)
-        :return: raw contents as chunks
+        Args:
+          name: Name of the item
+          etag: Optional etag (ignored)
+        Returns: raw contents as chunks
         """
         path = os.path.join(self.path, name)
         try:
             with open(path, "rb") as f:
                 return [f.read()]
-        except IOError as e:
-            if e.errno == errno.ENOENT:
-                raise KeyError
-            raise
+        except FileNotFoundError as exc:
+            raise KeyError(name) from exc
+        except IsADirectoryError as exc:
+            raise KeyError(name) from exc
 
     def _scan_uids(self):
         removed = set(self._fname_to_uid.keys())
         for (name, content_type, etag) in self.iter_with_etag():
             if name in removed:
                 removed.remove(name)
-            if name in self._fname_to_uid and self._fname_to_uid[name][0] == etag:
+            if (name in self._fname_to_uid
+                    and self._fname_to_uid[name][0] == etag):
                 continue
             fi = open_by_extension(
                 self._get_raw(name, etag), name, self.extra_file_handlers
@@ -165,21 +156,23 @@ class VdirStore(Store):
     ):
         """Import a single object.
 
-        :param name: name of the object
-        :param content_type: Content type
-        :param data: serialized object as list of bytes
-        :param message: Commit message
-        :param author: Optional author
-        :param replace_etag: optional etag of object to replace
-        :raise InvalidETag: when the name already exists but with different
-                            etag
-        :raise DuplicateUidError: when the uid already exists
-        :return: etag
+        Args:
+          name: name of the object
+          content_type: Content type
+          data: serialized object as list of bytes
+          message: Commit message
+          author: Optional author
+          replace_etag: optional etag of object to replace
+        Raises:
+          InvalidETag: when the name already exists but with different etag
+          DuplicateUidError: when the uid already exists
+        Returns: etag
         """
         if content_type is None:
             fi = open_by_extension(data, name, self.extra_file_handlers)
         else:
-            fi = open_by_content_type(data, content_type, self.extra_file_handlers)
+            fi = open_by_content_type(
+                data, content_type, self.extra_file_handlers)
         if name is None:
             name = str(uuid.uuid4())
             extension = MIMETYPES.guess_extension(content_type)
@@ -208,8 +201,9 @@ class VdirStore(Store):
     def iter_with_etag(self, ctag=None):
         """Iterate over all items in the store with etag.
 
-        :param ctag: Ctag to iterate for
-        :yield: (name, content_type, etag) tuples
+        Args:
+          ctag: Ctag to iterate for
+        Returns: iterator over (name, content_type, etag) tuples
         """
         for name in os.listdir(self.path):
             if name.endswith(".tmp"):
@@ -228,7 +222,7 @@ class VdirStore(Store):
     def create(cls, path: str) -> "VdirStore":
         """Create a new store backed by a Vdir on disk.
 
-        :return: A `VdirStore`
+        Returns: A `VdirStore`
         """
         os.mkdir(path)
         return cls(path)
@@ -237,44 +231,49 @@ class VdirStore(Store):
     def open_from_path(cls, path: str) -> "VdirStore":
         """Open a VdirStore from a path.
 
-        :param path: Path
-        :return: A `VdirStore`
+        Args:
+          path: Path
+        Returns: A `VdirStore`
         """
         return cls(path)
 
     def get_description(self):
         """Get extended description.
 
-        :return: repository description as string
+        Returns: repository description as string
         """
         return self.config.get_description()
 
     def set_description(self, description):
         """Set extended description.
 
-        :param description: repository description as string
+        Args:
+          description: repository description as string
         """
         self.config.set_description(description)
 
     def set_comment(self, comment):
         """Set comment.
 
-        :param comment: Comment
+        Args:
+          comment: Comment
         """
         raise NotImplementedError(self.set_comment)
 
     def get_comment(self):
         """Get comment.
 
-        :return: Comment
+        Returns: Comment
         """
         raise NotImplementedError(self.get_comment)
 
     def _read_metadata(self, name):
         try:
-            with open(os.path.join(self.path, name), "r") as f:
+            with open(os.path.join(self.path, name)) as f:
                 return f.read().strip()
-        except EnvironmentError:
+        except FileNotFoundError:
+            return None
+        except IsADirectoryError:
             return None
 
     def _write_metadata(self, name, data):
@@ -288,7 +287,7 @@ class VdirStore(Store):
     def get_color(self):
         """Get color.
 
-        :return: A Color code, or None
+        Returns: A Color code, or None
         """
         color = self._read_metadata("color")
         if color is not None:
@@ -311,23 +310,25 @@ class VdirStore(Store):
     def get_displayname(self):
         """Get display name.
 
-        :return: The display name, or None if not set
+        Returns: The display name, or None if not set
         """
         return self._read_metadata("displayname")
 
     def set_displayname(self, displayname):
         """Set the display name.
 
-        :param displayname: New display name
+        Args:
+          displayname: New display name
         """
         self._write_metadata("displayname", displayname)
 
     def iter_changes(self, old_ctag, new_ctag):
         """Get changes between two versions of this store.
 
-        :param old_ctag: Old ctag (None for empty Store)
-        :param new_ctag: New ctag
-        :return: Iterator over (name, content_type, old_etag, new_etag)
+        Args:
+          old_ctag: Old ctag (None for empty Store)
+          new_ctag: New ctag
+        Returns: Iterator over (name, content_type, old_etag, new_etag)
         """
         raise NotImplementedError(self.iter_changes)
 
@@ -338,12 +339,14 @@ class VdirStore(Store):
     def delete_one(self, name, message=None, author=None, etag=None):
         """Delete an item.
 
-        :param name: Filename to delete
-        :param message: Commit message
-        :param author: Optional author
-        :param etag: Optional mandatory etag of object to remove
-        :raise NoSuchItem: when the item doesn't exist
-        :raise InvalidETag: If the specified ETag doesn't match the curren
+        Args:
+          name: Filename to delete
+          message: Commit message
+          author: Optional author
+          etag: Optional mandatory etag of object to remove
+        Raises:
+          NoSuchItem: when the item doesn't exist
+          InvalidETag: If the specified ETag doesn't match the curren
         """
         path = os.path.join(self.path, name)
         if etag is not None:
@@ -355,10 +358,10 @@ class VdirStore(Store):
                 raise InvalidETag(name, etag, current_etag)
         try:
             os.unlink(path)
-        except EnvironmentError as e:
-            if e.errno == errno.ENOENT:
-                raise NoSuchItem(path)
-            raise
+        except FileNotFoundError as exc:
+            raise NoSuchItem(path) from exc
+        except IsADirectoryError as exc:
+            raise NoSuchItem(path) from exc
 
     def get_ctag(self):
         """Return the ctag for this store."""
@@ -367,7 +370,7 @@ class VdirStore(Store):
     def subdirectories(self):
         """Returns subdirectories to probe for other stores.
 
-        :return: List of names
+        Returns: List of names
         """
         ret = []
         for name in os.listdir(self.path):
