@@ -17,24 +17,24 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA  02110-1301, USA.
 
-"""WSGI wrapper for xandikos.
-"""
+"""WSGI wrapper for xandikos."""
 
 import posixpath
 
 from .web import WELLKNOWN_DAV_PATHS
 
 
-class WellknownRedirector(object):
+class WellknownRedirector:
     """Redirect paths under .well-known/ to the appropriate paths."""
 
-    def __init__(self, inner_app, dav_root):
+    def __init__(self, inner_app, dav_root) -> None:
         self._inner_app = inner_app
         self._dav_root = dav_root
 
     def __call__(self, environ, start_response):
         # See https://tools.ietf.org/html/rfc6764
-        path = posixpath.normpath(environ["SCRIPT_NAME"] + environ["PATH_INFO"])
+        path = posixpath.normpath(
+            environ["SCRIPT_NAME"] + environ["PATH_INFO"])
         if path in WELLKNOWN_DAV_PATHS:
             start_response("302 Found", [("Location", self._dav_root)])
             return []
